import logging
import os
from datetime import datetime
from hashlib import md5
from pathlib import Path

from PIL import ExifTags, Image

log = logging.getLogger(__name__)

EXIF_DATE_TIME_ORIG = "DateTimeOriginal"
EXIF_ORIENTATION = "Orientation"
EXIF_RATING = "Rating"
EXIF_GPS_INFO = "GPSInfo"

EXIF_GPS_INFO_LAT = "GPSLatitude"
EXIF_GPS_INFO_LAT_REF = "GPSLatitudeRef"
EXIF_GPS_INFO_LON = "GPSLongitude"
EXIF_GPS_INFO_LON_REF = "GPSLongitudeRef"


class ExifData:
    def __init__(self, path):
        self.reverse_exif_tags = {v: k for k, v in ExifTags.TAGS.items()}
        self.reverse_exif_gps_tags = {v: k for k, v in ExifTags.GPSTAGS.items()}
        image = Image.open(path)
        try:
            self._exif_data = image.getexif()
        except AttributeError:
            log.debug(f"No getexif function for file {path}. Trying _getexif")
            self._exif_data = image._getexif()
        if self._exif_data is None:
            log.info(f"No exif data found for file {path}")
        self.orig_date_time = self._get_date_time(path)
        self.orientation = int(self._get_from_exif(EXIF_ORIENTATION) or 1)
        self.rating = int(self._get_from_exif(EXIF_RATING) or -1)
        self._gps = self._get_from_exif(EXIF_GPS_INFO)
        self.lat = self._get_from_gps_exif(EXIF_GPS_INFO_LAT)
        self.lat_ref = self._get_from_gps_exif(EXIF_GPS_INFO_LAT_REF)
        self.long = self._get_from_gps_exif(EXIF_GPS_INFO_LON)
        self.long_ref = self._get_from_gps_exif(EXIF_GPS_INFO_LON_REF)

    @staticmethod
    def _convert_gps_tuple_to_decimal(value):
        if type(value) is tuple:
            try:
                return value[0] + value[1] / 60.0 + value[2] / 3600.0
            except Exception as e:
                log.warning(f"Cannot convert {value} to latitude decimal.", exc_info=e)
        else:
            return value

    @property
    def lat(self):
        return self._lat

    @lat.setter
    def lat(self, value):
        self._lat = self._convert_gps_tuple_to_decimal(value)

    @property
    def long(self):
        return self._long

    @long.setter
    def long(self, value):
        self._long = self._convert_gps_tuple_to_decimal(value)

    def _get_from_exif(self, tag: str):
        if tag in self.reverse_exif_tags:
            exif_tag = self.reverse_exif_tags[tag]
            if self._exif_data and exif_tag in self._exif_data:
                return self._exif_data[exif_tag]
        return None

    def _get_from_gps_exif(self, tag: str):
        if tag in self.reverse_exif_gps_tags:
            exif_gps_tag = self.reverse_exif_gps_tags[tag]
            if self._gps and exif_gps_tag in self._gps:
                return self._gps[exif_gps_tag]
        return None

    def _get_date_time(self, path: str):
        exif_dt = self._get_from_exif(EXIF_DATE_TIME_ORIG)
        if exif_dt:
            return datetime.strptime(exif_dt, "%Y:%m:%d %H:%M:%S")
        else:
            return None

    def __str__(self):
        return (
            f"Original Date Time: {self.orig_date_time}\n"
            f"Orientation: {self.orientation}\n"
            f"Rating: {self.rating}\n"
            f"GPS: {self._gps}"
        )


def lazy_property(fn):
    """
    Decorator that makes a property lazy-evaluated.
    """
    attr_name = "_lazy_" + fn.__name__

    @property
    def _lazy_property(self):
        if not hasattr(self, attr_name):
            setattr(self, attr_name, fn(self))
        return getattr(self, attr_name)

    return _lazy_property


class PictureFile:
    def __init__(self, path):
        self.path = path
        self.mtime = os.path.getmtime(path)

    @lazy_property
    def exif_data(self) -> ExifData:
        return ExifData(self.path)

    @lazy_property
    def hash(self):
        with open(self.path, "rb") as f:
            return md5(f.read()).hexdigest()

    def __str__(self):
        return (
            f"Picture:\n"
            f"File Path: {self.path}\n"
            f"Modification time: {self.mtime}"
            f"Exif Data:\n{self.exif_data}"
        )


def read_pictures_from_disk(pic_dir: str):
    extensions = [".png", ".jpg", ".jpeg"]  # TODO move this to config
    picture_dir = pic_dir
    for root, _, filenames in os.walk(picture_dir):
        for filename in filenames:
            if os.path.splitext(filename)[1].lower() in extensions:
                abs_path = str(Path(root, filename))
                yield PictureFile(abs_path)



import logging
import os
from datetime import datetime

from sqlalchemy.orm import Session

from pipictureframe.picdb.Database import (
    Database,
    LAST_DB_UPDATE_KEY_STR,
    LAST_DB_UPDATE_FMT_STR,
)
from pipictureframe.picdb.DbObjects import PictureData, Metadata
from pipictureframe.utils.PictureReader import read_pictures_from_disk, PictureFile

log = logging.getLogger(__name__)


def update_pictures_in_db(pic_dir: str, connections_str: str):
    pic_file_gen = read_pictures_from_disk(pic_dir)
    db = Database(connections_str)

    session = db.get_session()
    db_changed = _clean_db(session)
    db_changed = _add_and_update_pics(pic_file_gen, session) or db_changed
    if db_changed:
        update_obj = Metadata(
            LAST_DB_UPDATE_KEY_STR, datetime.now().strftime(LAST_DB_UPDATE_FMT_STR)
        )
        session.merge(update_obj)
        session.commit()
    session.close()


def _clean_db(session) -> bool:
    db_changed = False
    all_pics = session.query(PictureData).all()
    for pic in all_pics:
        if not os.path.exists(pic.absolute_path):
            session.delete(pic)
            session.commit()
            db_changed = True
    return db_changed


def _add_and_update_pics(pic_file_gen, session) -> bool:
    db_changed = False
    for pic_file in pic_file_gen:
        pic_by_path = _get_pic_by_path(session, pic_file)
        # Is present
        if pic_by_path:
            # Has been modified
            if pic_by_path.mtime < pic_file.mtime:
                log.debug(f"Updated timestamp detected for {pic_file.path}")
                pic_data = PictureData.from_picture_file(pic_file)
                pic_by_hash = _get_pic_by_hash(session, pic_data.hash_id)
                # If modified but hash has not changed
                if pic_by_hash:
                    log.debug(f"Metadata but not hash has changed for {pic_file.path}")
                    session.merge(pic_data)
                # If hash has changed
                else:
                    log.debug(f"Hash has changed for {pic_file.path}")
                    session.delete(pic_by_path)
                    session.add(pic_data)
        # If not present
        else:
            pic_data = PictureData.from_picture_file(pic_file)
            pic_by_hash = _get_pic_by_hash(session, pic_data.hash_id)
            if pic_by_hash:
                log.debug(
                    f"Picture has moved from {pic_by_hash.absolute_path} to {pic_data.absolute_path}"
                )
                session.merge(pic_data)
            else:
                log.debug(f"Picture {pic_file.path} will be added to the database.")
                session.add(pic_data)
        session.commit()
        db_changed = True
    return db_changed


def _get_pic_by_path(session: Session, pic_file: PictureFile) -> PictureData:
    q = session.query(PictureData).filter(PictureData.absolute_path == pic_file.path)
    return q.first()


def _get_pic_by_hash(session: Session, hash_id: str) -> PictureData:
    q = session.query(PictureData).filter(PictureData.hash_id == hash_id)
    return q.first()



from datetime import datetime

from pipictureframe.utils.PictureReader import PictureFile


class PictureData:
    def __init__(
        self,
        hash_id: str,
        abs_path: str,
        mtime: int,
        orig_datetime: datetime,
        orientation: int,
        rating: int,
        lat_ref: str,
        lat: float,
        long_ref: str,
        long: float,
        times_shown: int = 0,
    ):
        self.hash_id = hash_id
        self.absolute_path = abs_path
        self.mtime = mtime
        self.orig_date_time = orig_datetime
        self.orientation = orientation
        self.rating = rating
        self.lat_ref = lat_ref
        self.lat = lat
        self.long_ref = long_ref
        self.long = long
        self.times_shown = times_shown

    def get_date_time(self):
        if self.orig_date_time:
            return self.orig_date_time
        else:
            return datetime.fromtimestamp(self.mtime)

    @staticmethod
    def from_picture_file(pic_file: PictureFile):
        pic_data = PictureData(
            pic_file.hash,
            pic_file.path,
            pic_file.mtime,
            pic_file.exif_data.orig_date_time,
            pic_file.exif_data.orientation,
            pic_file.exif_data.rating,
            pic_file.exif_data.lat_ref,
            pic_file.exif_data.lat,
            pic_file.exif_data.long_ref,
            pic_file.exif_data.long,
        )
        return pic_data

    def __repr__(self):
        return str(self.__dict__)


class Metadata:
    key = None

    def __init__(self, key: str, value: str):
        self.key = key
        self.value = value

    def __repr__(self):
        return str(self.__dict__)



#!/usr/bin/python3

""" Simplified slideshow system using ImageSprite.
    Also has a minimal use of PointText and TextBlock system with reduced codepoints
and reduced grid_size to give better resolution for large characters.

USING exif info to rotate images
"""

import logging
import multiprocessing as mp
import os
import time
from logging.handlers import RotatingFileHandler

from pipictureframe.pi3dfuncs import Pi3dFuncs
from pipictureframe.picdb.Database import Database
from pipictureframe.picdb.PictureUpdater import update_pictures_in_db, PictureData
from pipictureframe.utils.Config import setup_parser, Config
from pipictureframe.utils.NextPictureManager import NextPictureManager

log = logging.getLogger()
log.setLevel(logging.DEBUG)


def start_pic_loading_process(conf: Config):
    pic_load_proc = mp.Process(
        target=update_pictures_in_db, args=(conf.pic_dir, conf.db_connection_str)
    )
    pic_load_proc.daemon = True
    pic_load_proc.start()
    return pic_load_proc


def load_next_slide(pi3dfuncs: Pi3dFuncs, cur_pic: PictureData, config):
    pi3dfuncs.copy_fg_to_bg()
    pi3dfuncs.load_fg(cur_pic, config)
    pi3dfuncs.set_textures()


def check_for_changes(last_file_change, pic_dir):
    update = False
    for root, _, _ in os.walk(pic_dir):
        mod_tm = os.stat(root).st_mtime
        if mod_tm > last_file_change:
            last_file_change = mod_tm
            update = True
    log.debug(f"Checked for updates to picture folder. Result = {update}")
    return update


class LoopControlVars:
    def __init__(self, config):
        self.cur_time = time.time()
        self.next_pic_time = self.cur_time + config.time_delay
        self.text_start_time = self.cur_time
        self.folder_recheck_time = self.cur_time + config.check_dir_tm
        # Initially pretend a transition has begun
        self.fg_alpha = 0
        self.text_alpha = 0
        self.transition_running = True

    def __repr__(self):
        return str(self.__dict__)


def main():
    parser = setup_parser()
    args = parser.parse_args()
    config = Config(args)

    # Configure logging
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    ch = logging.StreamHandler()
    ch.setLevel(config.log_level)
    ch.setFormatter(formatter)
    log.addHandler(ch)
    if config.debug_log_file:
        try:
            dfh = RotatingFileHandler(
                config.debug_log_file,
                maxBytes=10 * 1024 * 1024,
                backupCount=5,
                encoding="utf-8",
            )
            dfh.setLevel(logging.DEBUG)
            dfh.setFormatter(formatter)
            log.addHandler(dfh)
        except Exception as e:
            log.error(
                f"Could not open debug log file {config.debug_log_file}.", exc_info=e
            )

    log.debug(f"Starting application with following config:\n{config}")

    db = Database(config.db_connection_str)

    pic_loading_proc = start_pic_loading_process(config)

    pi3dfuncs = Pi3dFuncs(config)

    # Set values for first slide a part of ctor
    lcv = LoopControlVars(config)
    log.debug(f"Initialized loop control variables:\n{lcv}")

    npm = NextPictureManager(config, pic_loading_proc, db)
    cur_pic = npm.get_next_picture()

    # Load first picture into fg
    pi3dfuncs.load_fg(cur_pic, config)
    # Immediately copy it to bg
    pi3dfuncs.copy_fg_to_bg()
    pi3dfuncs.set_fg_bg_ratio(lcv.fg_alpha)
    pi3dfuncs.set_textures()
    pi3dfuncs.prepare_text(cur_pic, config)

    # TODO implement pause functionality
    while pi3dfuncs.display_loop_running():
        lcv.cur_time = time.time()
        if lcv.cur_time > lcv.next_pic_time and not lcv.transition_running:
            log.debug(f"Starting transition.\n{lcv}")
            lcv.text_start_time = lcv.cur_time
            lcv.next_pic_time = lcv.cur_time + config.time_delay
            pi3dfuncs.prepare_text(cur_pic, config)
            lcv.transition_running = True

        if lcv.transition_running:
            lcv.fg_alpha += config.delta_alpha
            lcv.text_alpha = lcv.fg_alpha
            if lcv.fg_alpha > 1.0:
                log.debug(f"Transition finished.\n{lcv}")
                # When the transition ends copy the slide shown to bg and load the next slide into fg
                lcv.transition_running = False
                cur_pic = npm.get_next_picture()
                load_next_slide(pi3dfuncs, cur_pic, config)
                lcv.fg_alpha = 0
            pi3dfuncs.set_fg_bg_ratio(lcv.fg_alpha)

        pi3dfuncs.draw_slide()

        # Text shouldn't be shown anymore, min is needed if show text time is longer than time delay
        if lcv.cur_time > lcv.text_start_time + min(
            config.show_text_tm, config.time_delay - 1
        ):
            if lcv.text_alpha > 0:
                lcv.text_alpha -= config.delta_alpha
            else:
                lcv.text_alpha = 0
            pi3dfuncs.set_text_alpha_and_draw(lcv.text_alpha)
        elif lcv.transition_running:
            pi3dfuncs.set_text_alpha_and_draw(lcv.text_alpha)
        else:
            pi3dfuncs.set_text_alpha_and_draw(1)

        # Also check that no transition is running to avoid artefacts:
        if (
            lcv.cur_time > lcv.folder_recheck_time
            and lcv.fg_alpha == 0
            and lcv.text_alpha == 0
        ):
            lcv.folder_recheck_time = lcv.cur_time + config.check_dir_tm
            if not pic_loading_proc.is_alive():
                pic_loading_proc = start_pic_loading_process(config)
                npm.reload_pictures()

        # TODO Fix keyboard input
        if config.keyboard:
            k = pi3dfuncs.keyboard.read()
            if k == 27:  # ESC
                break
            elif k == ord(" "):
                pass
                # paused = not paused
            elif k != -1:  # Next slide
                lcv.next_pic_time = lcv.cur_time

    if config.keyboard:
        pi3dfuncs.keyboard.close()
    pic_loading_proc.join()
    # TODO move display destroy etc to pi3dfuncs function
    pi3dfuncs._display.destroy()
    db.close()


if __name__ == "__main__":
    main()



from datetime import datetime
from typing import List
from unittest.mock import Mock, PropertyMock

from pipictureframe.picdb.DbObjects import Metadata
from pipictureframe.picdb.PictureUpdater import PictureData
from pipictureframe.picdb.Database import LAST_DB_UPDATE_FMT_STR


def get_config_mock():
    config = Mock()
    config.shuffle = PropertyMock()
    config.shuffle = False
    config.min_rating = PropertyMock()
    config.min_rating = None
    config.max_rating = PropertyMock()
    config.max_rating = None
    return config


def get_db_mock(return_pics: List[PictureData] = list()):
    query = Mock()
    query.filter = Mock(return_value=query)
    query.all = Mock(return_value=return_pics)
    query.one = Mock(
        return_value=Metadata("x", datetime.now().strftime(LAST_DB_UPDATE_FMT_STR))
    )
    session = Mock()
    session.query = Mock(return_value=query)
    db = Mock()
    db.get_session = Mock(return_value=session)
    return db



import argparse
import os
import logging

log = logging.getLogger(__name__)


def str_to_tuple(x):
    return tuple(float(v) for v in x.replace("(", "").replace(")", "").split(","))


def parse_show_text(txt):
    show_text = 0
    txt = txt.lower()
    if "name" in txt:
        show_text |= 1
    if "date" in txt:
        show_text |= 2
    if "location" in txt:
        show_text |= 4
    if "folder" in txt:
        show_text |= 8
    return show_text


def setup_parser():
    parser = argparse.ArgumentParser("start running a picture frame")
    vpo = parser.add_argument_group("visual picture options")
    vpo.add_argument(
        "-a",
        "--blur_amount",
        default=12,
        type=float,
        help="larger values than 12 will increase processing load quite a bit",
    )
    vpo.add_argument(
        "-b",
        "--blur_edges",
        action="store_true",
        help="use blurred version of image to fill edges - will override FIT = False",
    )
    vpo.add_argument(
        "-e",
        "--edge_alpha",
        default=0.5,
        type=float,
        help="background colour at edge. 1.0 would show reflection of image",
    )
    vpo.add_argument(
        "-g",
        "--background",
        default=(0.2, 0.2, 0.3, 1.0),
        type=str_to_tuple,
        help="RGBA to fill edges when fitting",
    )
    vpo.add_argument(
        "-j",
        "--blend_type",
        default="blend",
        choices=["blend", "burn", "bump"],
        help="type of blend the shader can do",
    )
    vpo.add_argument(
        "-f", "--fit", action="store_true", help="shrink to fit screen i.e. don't crop"
    )
    vpo.add_argument(
        "-v",
        "--time_delay",
        default=8.0,
        type=float,
        help="time between consecutive slide starts",
    )
    vpo.add_argument(
        "-w",
        "--fade_time",
        default=1.0,
        type=float,
        help="change time during which slides overlap",
    )
    vpo.add_argument(
        "-z",
        "--blur_zoom",
        default=1.0,
        type=float,
        help="must be >= 1.0 which expands the background to just fill the space around the image",
    )
    vpo.add_argument(
        "--no_auto_resize",
        dest="auto_resize",
        action="store_false",
        help="Set this if you want to use 4K resolution on Raspberry Pi 4."
        " You should ensure your images are the correct size for the display",
    )

    vto = parser.add_argument_group("visual text options")
    vto.add_argument(
        "-s",
        "--show_text_tm",
        default=6.0,
        type=float,
        help="time to show text over the image",
    )
    vto.add_argument(
        "--show_date_fm", default="%b %d, %Y", help="format to show date over the image"
    )
    vto.add_argument("--show_text_sz", default=40, type=int, help="text character size")
    vto.add_argument(
        "--show_text",
        default="date folder location",
        help="show text, include combination of words: name, date, location",
    )
    vto.add_argument(
        "--text_width",
        default=90,
        type=int,
        help="number of character before breaking into new line",
    )
    # TODO reimplement
    vto.add_argument("--load_geoloc", action="store_true", help="load geolocation code")
    vto.add_argument(
        "--geo_key",
        default="picture_frame_hello",
        help="set the Nominatim key - change to something unique to you",
    )
    vto.add_argument(
        "--geo_path",
        default="/home/pi/PictureFrame2020gpsdata.txt",
        help="set the local file to store data from geopy - ignored if --load_geoloc is not true",
    )
    vto.add_argument(
        "--geo_zoom",
        default=10,
        type=int,
        help="Level of address detail(3=country...18=building): 3,5,8,10,14,16,17,18",
    )

    fo = parser.add_argument_group("filter options")
    fo.add_argument(
        "--min_rating",
        default=None,
        type=int,
        help="Minimum rating of displayed pictures",
    )
    fo.add_argument(
        "--max_rating",
        default=None,
        type=int,
        help="Maximum rating of displayed pictures",
    )
    # TODO add filters for date_before and date_after

    pfo = parser.add_argument_group("picture file options")
    pfo.add_argument(
        "-c",
        "--check_dir_tm",
        default=1800.0,
        type=float,
        help="time in seconds between checking if the image directory has changed",
    )
    pfo.add_argument(
        "-r",
        "--reshuffle_num",
        default=1,
        type=int,
        help="times through before reshuffling",
    )
    pfo.add_argument(
        "-x", "--shuffle", action="store_true", help="shuffle on loading image files"
    )
    pfo.add_argument(
        "--shuffle_weight",
        default=0,
        type=float,
        help="Determines the weight with which the number of times a picture has already been shown,"
        " influces the probablity for it to be picked in the future."
        " 0 Means no influce, 1 means about 50 percent influence and 100 means complete influence.",
    )

    mo = parser.add_argument_group("misc options")
    mo.add_argument(
        "-l",
        "--log_level",
        choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
        default="WARNING",
        help="Set the logging level",
    )
    mo.add_argument(
        "--debug_log_file",
        default=None,
        help="Set path to a file in which to record all debug log messages.",
    )
    mo.add_argument(
        "-o", "--font_file", default="pipictureframe/fonts/NotoSans-Regular.ttf"
    )
    mo.add_argument("-p", "--pic_dir", default="/home/pi/Pictures")
    mo.add_argument(
        "--db_connection_string",
        default="sqlite:////home/pi/picture_db.db",
        help="SqlAlchemy is used so not only other locations but"
        " also other databases are possible.",
    )
    mo.add_argument("-q", "--shader", default="pipictureframe/shaders/blend_new")
    mo.add_argument("--locale", default="en_US.utf8", help="set the locale")
    mo.add_argument("--fps", default=20.0, type=float)
    mo.add_argument(
        "--display_x",
        default=0,
        type=int,
        help="offset from left of screen (can be negative)",
    )
    mo.add_argument(
        "--display_y",
        default=0,
        type=int,
        help="offset from top of screen (can be negative)",
    )
    mo.add_argument(
        "--display_w",
        default=None,
        type=int,
        help="width of display surface (None will use max returned by hardware)",
    )
    mo.add_argument(
        "--display_h", default=None, type=int, help="height of display surface"
    )
    # TODO reimplement
    mo.add_argument(
        "-k", "--keyboard", action="store_true", help="Enable keyboard interaction. "
    )
    # TODO possibly reimplement
    # parse.add_argument("-i", "--no_files_img",
    #                    default="/home/pi/pi3d_demos/PictureFrame2020img.jpg",
    #                    help="image to show if none selected")
    # parse.add_argument("-m", "--use_mqtt",      default=True)
    # parse.add_argument(      "--mqtt_server",   default="localhost")
    # parse.add_argument(      "--mqtt_port",     default=1883, type=int)
    # parse.add_argument(      "--mqtt_login",    default="")
    # parse.add_argument(      "--mqtt_password", default="")
    # parse.add_argument(      "--mqtt_id",       default="frame",
    # help="prepended onto all the message strings with a / separator added")
    # parse.add_argument("-y", "--subdirectory",  default="", help="subdir of pic_dir - can be changed by MQTT")

    return parser


def convert_to_absolute_path(rel_path):
    if os.path.isfile(rel_path):
        return os.path.abspath(rel_path)
    else:
        new_path = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), "../..", rel_path
        )
        if os.path.isfile(new_path):
            return new_path
        else:
            log.error(f"Neither {rel_path} nor {new_path} could be found.")
    return None


class Config:
    def __init__(self, args):
        self.blur_amount = args.blur_amount
        self.blur_edges = args.blur_edges
        self.edge_alpha = args.edge_alpha
        self.background = args.background

        blend_options = {
            "blend": 0.0,
            "burn": 1.0,
            "bump": 2.0,
        }  # that work with the blend_new fragment shader
        self.blend_type = blend_options[args.blend_type]
        self.fit = args.fit
        self.time_delay = args.time_delay
        self.fade_time = args.fade_time
        self.blur_zoom = max(args.blur_zoom, 1.0)
        self.auto_resize = args.auto_resize

        self.show_text_tm = args.show_text_tm
        self.show_date_fm = args.show_date_fm
        self.show_text_sz = args.show_text_sz
        self.show_text = parse_show_text(args.show_text)
        self.text_width = args.text_width
        self.load_geoloc = args.load_geoloc
        self.geo_key = args.geo_key
        self.geo_path = args.geo_path
        self.geo_zoom = args.geo_zoom

        self.min_rating = args.min_rating
        self.max_rating = args.max_rating

        self.check_dir_tm = args.check_dir_tm
        self.reshuffle_num = args.reshuffle_num
        self.shuffle = args.shuffle
        self.shuffle_weight = args.shuffle_weight

        self.log_level = args.log_level
        self.debug_log_file = args.debug_log_file
        self.fps = args.fps
        self.font_file = convert_to_absolute_path(args.font_file)
        self.pic_dir = args.pic_dir
        # This is needed because the path to the shader is given without the .fs/.vs extension.
        self.shader = convert_to_absolute_path(args.shader + ".fs")[:-3]
        self.local = args.locale
        self.display_x = args.display_x
        self.display_y = args.display_y
        self.display_w = args.display_w
        self.display_h = args.display_h
        self.keyboard = args.keyboard

        self.delta_alpha = 1.0 / (self.fps * self.fade_time)
        # limit to 121 ie 11x11 grid_size
        self.codepoints = (
            "1234567890AÄÀÆÅÃBCÇDÈÉÊEËFGHIÏÍJKLMNÑOÓÖÔŌØPQRSTUÚÙÜVWXYZ"
            "aáàãæåäbcçdeéèêëfghiíïjklmnñoóôōøöpqrsßtuúüvwxyz., _-+*()&/`´'•"
        )
        self.db_connection_str = args.db_connection_string

    def __repr__(self):
        return str(self.__dict__)



import logging
from datetime import datetime

import sqlalchemy
from sqlalchemy import Column, String, Integer, DateTime, Float
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm import sessionmaker, Session
from sqlalchemy.orm.exc import UnmappedClassError

from pipictureframe.picdb.DbObjects import PictureData, Metadata

log = logging.getLogger(__name__)


VERSION_STRING = "version"
CURRENT_DB_VERSION = 1
LAST_DB_UPDATE_KEY_STR = "last_db_update"
LAST_DB_UPDATE_FMT_STR = "%Y-%m-%d %H:%M:%S"


class Database:
    def __init__(self, connection_string: str, echo=False):
        self.engine = sqlalchemy.create_engine(connection_string, echo=echo)
        self.connection = self.engine.connect()
        self.alchemy_metadata = sqlalchemy.MetaData(bind=self.engine)
        self.sm = sessionmaker()
        self._initialize_db_picture_mapping()
        self._initizalize_metadata_table()
        self.alchemy_metadata.create_all()
        self.create_inital_metadata_objects()
        self.version = self._get_db_version()

    def create_inital_metadata_objects(self):
        try:
            session = self.get_session()
            session.add(Metadata(VERSION_STRING, str(CURRENT_DB_VERSION)))
            session.add(
                Metadata(
                    LAST_DB_UPDATE_KEY_STR,
                    datetime.now().strftime(LAST_DB_UPDATE_FMT_STR),
                )
            )
            session.commit()
            log.debug("Setting up initial metadata objects successful.")
        except IntegrityError:
            log.debug("Metadata objects already set up. Probably already existing db.")
            pass
        finally:
            session.close()

    def _get_db_version(self):
        session = self.get_session()
        version_obj: Metadata = (
            session.query(Metadata).filter(Metadata.key == VERSION_STRING).one()
        )
        version = version_obj.value
        session.close()
        log.debug(f"Retrieved db version = {version}")
        return int(version)

    def _initialize_db_picture_mapping(self):
        try:
            sqlalchemy.orm.class_mapper(PictureData)
        except UnmappedClassError:  # map class as it is not yet mapped.
            pic_data_map = sqlalchemy.Table(
                "pictures",
                self.alchemy_metadata,
                Column("hash_id", String(50), primary_key=True),
                Column("absolute_path", String(256)),
                Column("mtime", Integer),
                Column("orig_date_time", DateTime),
                Column("orientation", Integer),
                Column("rating", Integer),
                Column("lat_ref", String(1)),
                Column("lat", Float),
                Column("long_ref", String(1)),
                Column("long", Float),
                Column("times_shown", Integer),
            )
            sqlalchemy.orm.mapper(PictureData, pic_data_map)

    def _initizalize_metadata_table(self):
        try:
            sqlalchemy.orm.class_mapper(Metadata)
        except UnmappedClassError:
            metadata_map = sqlalchemy.Table(
                "metadata",
                self.alchemy_metadata,
                Column("key", String(256), primary_key=True),
                Column("value", String(1024)),
            )
            sqlalchemy.orm.mapper(Metadata, metadata_map)

    def get_session(self) -> Session:
        return self.sm()

    def close(self):
        self.sm.close_all()
        self.connection.close()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pipictureframe",
    version="0.9.5",
    packages=setuptools.find_packages(),
    url="https://github.com/cornim/pipictureframe",
    license="GPLv3",
    author="Dr. Cornelius Mund",
    author_email="",
    description="A program to use a raspberry pi and a monitor as a digital picture frame.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    scripts=["pi-picture-frame"],
    install_requires=[
        "pi3d~=2.41",
        "numpy>=1.19.5",
        "Pillow~=8.1.0",
        "geopy~=2.1.0",
        "sqlalchemy~=1.3.23",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: POSIX :: Linux",
    ],
)
